// -----------------------------------------------------------------------------
// PARAMETERS:
// -----------------------------------------------------------------------------
//transforms
float4x4 tWVP: WORLDVIEWPROJECTION ;

texture TexPrev <string uiname="Previous Frame Data Texture";>;
sampler SampPrev = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexPrev);          //apply a texture to the sampler
    MipFilter = none;               //sampler states
    MinFilter = none;
    MagFilter = none;
};

texture TexReset <string uiname="PP Reset Texture";>;
sampler SampReset = sampler_state    
{
    Texture   = (TexReset);          
    MipFilter = none;         
    MinFilter = none;
    MagFilter = none;
};

texture TexVel <string uiname="PP XYZ Velocity Texture";>;
sampler SampVel = sampler_state    
{
    Texture   = (TexVel);          
    MipFilter = none;         
    MinFilter = none;
    MagFilter = none;
};

float UpTime;
float3 ResetPos <string uiname="Emitter Position";>;

struct vs2ps
{
    float4 Pos : POSITION ;
    float4 TexCd : TEXCOORD0 ;
};

// -----------------------------------------------------------------------------
// VERTEXSHADERS
// -----------------------------------------------------------------------------
vs2ps VS(
    float4 Pos : POSITION ,
    float4 TexCd : TEXCOORD0 )
{
    vs2ps Out = (vs2ps)0;      //inititalize all fields of output struct with 0
    Out.Pos = mul(Pos, tWVP);  //transform position
    Out.TexCd = TexCd;

    return Out;
}

// -----------------------------------------------------------------------------
// PIXELSHADERS:
// -----------------------------------------------------------------------------
float4 PS(vs2ps In): COLOR
{
    // take the RGBA values from the last frame texture
    float4 lastFrame = tex2D(SampPrev, In.TexCd);
    // get the reset bang info from the alpha channel of the Reset Texture
    bool reset = tex2D(SampReset, In.TexCd).a > 0.5; 
    // get the XYZ velocity values from the RGB channels of Velocity Texture
    float3 vel = tex2D(SampVel, In.TexCd).rgb;
    // new RGBA data
    float4 newData = 0;
   
    /////// GPU CYCLE:
    if(reset)  //set the reset position to RGB and the birth time to A
      { newData = float4(ResetPos, UpTime); } 
    else  //get the old xyz and add xyz velocity; pass the birthTime to A
      { newData = float4(lastFrame.rgb + vel, lastFrame.a); }
 
    return newData;
}

// -----------------------------------------------------------------------------
// TECHNIQUES:
// -----------------------------------------------------------------------------
technique RGBA_Cycle
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PS();
    }
}
